@@ -1,10 +1,15 @@
 import collections
 from functools import partial
 import string
+import subprocess
+import sys
+import textwrap
 
 import numpy as np
 import pytest
 
+import pandas.util._test_decorators as td
+
 import pandas as pd
 from pandas import Series
 import pandas._testing as tm
@@ -49,36 +54,36 @@ def test_all_not_none():
 
 
 def test_random_state():
-    import numpy.random as npr
-
     # Check with seed
     state = com.random_state(5)
-    assert state.uniform() == npr.RandomState(5).uniform()
+    assert state.uniform() == np.random.RandomState(5).uniform()
 
     # Check with random state object
-    state2 = npr.RandomState(10)
-    assert com.random_state(state2).uniform() == npr.RandomState(10).uniform()
+    state2 = np.random.RandomState(10)
+    assert com.random_state(state2).uniform() == np.random.RandomState(10).uniform()
 
     # check with no arg random state
     assert com.random_state() is np.random
 
     # check array-like
     # GH32503
-    state_arr_like = npr.randint(0, 2 ** 31, size=624, dtype="uint32")
+    state_arr_like = np.random.default_rng(None).integers(
+        0, 2**31, size=624, dtype="uint32"
+    )
     assert (
         com.random_state(state_arr_like).uniform()
-        == npr.RandomState(state_arr_like).uniform()
+        == np.random.RandomState(state_arr_like).uniform()
     )
 
     # Check BitGenerators
     # GH32503
     assert (
-        com.random_state(npr.MT19937(3)).uniform()
-        == npr.RandomState(npr.MT19937(3)).uniform()
+        com.random_state(np.random.MT19937(3)).uniform()
+        == np.random.RandomState(np.random.MT19937(3)).uniform()
     )
     assert (
-        com.random_state(npr.PCG64(11)).uniform()
-        == npr.RandomState(npr.PCG64(11)).uniform()
+        com.random_state(np.random.PCG64(11)).uniform()
+        == np.random.RandomState(np.random.PCG64(11)).uniform()
     )
 
     # Error for floats or strings
@@ -213,3 +218,72 @@ class TestIsBoolIndexer:
         result = df[frozen]
         expected = df[[]]
         tm.assert_frame_equal(result, expected)
+
+
+@pytest.mark.parametrize("with_exception", [True, False])
+def test_temp_setattr(with_exception):
+    # GH#45954
+    ser = Series(dtype=object)
+    ser.name = "first"
+    # Raise a ValueError in either case to satisfy pytest.raises
+    match = "Inside exception raised" if with_exception else "Outside exception raised"
+    with pytest.raises(ValueError, match=match):
+        with com.temp_setattr(ser, "name", "second"):
+            assert ser.name == "second"
+            if with_exception:
+                raise ValueError("Inside exception raised")
+        raise ValueError("Outside exception raised")
+    assert ser.name == "first"
+
+
+@pytest.mark.single_cpu
+def test_str_size():
+    # GH#21758
+    a = "a"
+    expected = sys.getsizeof(a)
+    pyexe = sys.executable.replace("\\", "/")
+    call = [
+        pyexe,
+        "-c",
+        "a='a';import sys;sys.getsizeof(a);import pandas;print(sys.getsizeof(a));",
+    ]
+    result = subprocess.check_output(call).decode()[-4:-1].strip("\n")
+    assert int(result) == int(expected)
+
+
+@pytest.mark.single_cpu
+def test_bz2_missing_import():
+    # Check whether bz2 missing import is handled correctly (issue #53857)
+    code = """
+        import sys
+        sys.modules['bz2'] = None
+        import pytest
+        import pandas as pd
+        from pandas.compat import get_bz2_file
+        msg = 'bz2 module not available.'
+        with pytest.raises(RuntimeError, match=msg):
+            get_bz2_file()
+    """
+    code = textwrap.dedent(code)
+    call = [sys.executable, "-c", code]
+    subprocess.check_output(call)
+
+
+@td.skip_if_installed("pyarrow")
+@pytest.mark.parametrize("module", ["pandas", "pandas.arrays"])
+def test_pyarrow_missing_warn(module):
+    # GH56896
+    response = subprocess.run(
+        [sys.executable, "-c", f"import {module}"],
+        capture_output=True,
+        check=True,
+    )
+    msg = """
+Pyarrow will become a required dependency of pandas in the next major release of pandas (pandas 3.0),
+(to allow more performant data types, such as the Arrow string type, and better interoperability with other libraries)
+but was not found to be installed on your system.
+If this would cause problems for you,
+please provide us feedback at https://github.com/pandas-dev/pandas/issues/54466
+"""  # noqa: E501
+    stderr_msg = response.stderr.decode("utf-8")
+    assert msg in stderr_msg, stderr_msg
